
/***************************************************************************
                          downloads.h  -  description
                             -------------------
    begin                : Sun Feb 9 2003
    copyright            : (C) 2003 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _ed2k_gui_downloads_h_included_
#define _ed2k_gui_downloads_h_included_

#include "progressbar.h"
#include <gtk/gtk.h>
#include <time.h>

/* the Download structure contains mostly the 'dynamic' data.
 * The 'static' stuff that does not change very often
 * (name, size, prio, extension, format) stays in the model.
 */

struct _Download
{
	gchar                 *name;              /* the filename as the core knows it, in locale  */

	gchar                 *name_utf8;         /* the filename as the core knows it, in UTF8   */
	gchar                 *name_collate_key;

	gchar                 *name_nojunk_utf8;  /* the filename without junk in UTF8 (generated on demand) */
	gchar                 *name_nojunk_collate_key;

	guint32                size;

//	guint                  partnum; /* TODO: get rid of this, use string */

	gchar                 *previewname; /* path to temp file, different according to core version*/

	guint8                 status;
	gchar                 *status_completing_string;  /* set if completing + local core to show completing progress */

	guint8                 priority;
	guint32                transfered;
	gfloat                 transfered_percent;
	guint32                left;
	gfloat                 speed;
	gint                   secs_left;
	guint                  avail; /* availability in percent */
	guint                  sources;

	time_t                 last_complete;
	time_t                 last_downloading;


	GdkPixbuf             *progressbar;  /* is re-drawn on demand in the data cell function */
	progressbarGap        *gaps_data;
	time_t                 last_gotten_gaps;

	guint                  min_sources;  /* the minimum number of sources as the gapslist tells us */

	gboolean               cancelled;
	gboolean               corrupted;
	gboolean               done; /* download is done, either completed or corrupted */

	time_t                 time_added;

	guint8                 hash[16];
	guint32                id;

	gchar                 *ext;

	GtkTreeIter            iter; /* standard GtkListStore has persistent iters */
};

typedef struct _Download Download;



enum
{
 STATUS_HASHING=0,
 STATUS_QUEUED=1,
 STATUS_LOOKING,
 STATUS_DOWNLOADING,
 STATUS_PAUSED,
 STATUS_IDS,
 STATUS_NOSRCS,
 STATUS_DONE,
 STATUS_HASHING2,
 STATUS_ERRLOADING,
 STATUS_COMPLETING,
 STATUS_COMPLETE,
 STATUS_CORRUPTED,
 STATUS_ERRHASHING,
 STATUS_TRANSFERRING,
 STATUS_RESUMING,
 STATUS_PAUSING,
 STATUS_LAST
} ;	/* RESUMING and PAUSING are GUI status only */

enum
{
 DL_PRIO_LOW = 0,
 DL_PRIO_NORMAL,
 DL_PRIO_HIGH,
 DL_PRIO_HIGHEST,
} ;

extern gfloat    total_downloaded;
extern time_t    seconds_download_was_nonzero;

void             downloads_view_save_settings (void);

void             downloads_cleanup (void);

void             downloads_add_download ( const guint8 *hash,
                                          const gchar  *name,
                                          guint32       size,
                                          const gchar  *previewname,
                                          guint8        priority,
                                          guint32       id); /* used from v49.5 to map status <-> download */


gboolean         downloads_remove_download ( const guint8  *hash,
                                             gchar        **name,
                                             gboolean      *corrupted,
                                             gboolean      *cancelled,
                                             guint         *size );

void             download_remove (Download *dl);


void             downloads_set_download_status ( guint32 id,
                                                 guint8 status,
                                                 gfloat speed,
                                                 guint32 trans,
                                                 guint8 avail,
                                                 guint8 sources);

gboolean         downloads_set_download_gaplist ( const guint8 *hash,
                                                  progressbarGap *pbgaps);

gboolean         downloads_set_download_gaplist_from_id ( guint32 id, progressbarGap *pbgaps); /* > v49.12 */

GSList          *downloads_get_all (void);

guint            downloads_get_completed_items_count (void);

GSList          *downloads_get_selected (void);

Download        *downloads_get_download_from_hash (const guint8 *filehash);

void             downloads_update_view_appearance (void);

gint             downloads_get_num (void);

void             downloads_pause_all (gfloat space_left_on_device);

void             downloads_pause_and_resume_all_active_downloads (void);

void             downloads_update_rules_hint (void);

GtkWidget       *downloads_page_create (void);

void             downloads_page_set_free_space (gfloat needed, gfloat temp, gfloat incoming);

void             downloads_page_set_file_descriptor_usage (glong core_open, glong user_max);


#endif


