/***************************************************************************
                             downloads_popup.c  -  description
                             ---------------------------------
    begin                : Sun Feb 9 2003
    copyright            : (C) 2003 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* TODO: replace numeric values for priority (0-3) with enum values */

#include <gdk/gdkkeysyms.h>

#include "blacklist.h"
#include "downloads.h"
#include "downloads_popup.h"
#include "global.h"
#include "guicoreconn.h"  /* for CORE_IS_LOCAL */
#include "gui_flags.h"    /* for IS_OVERNET or IS_HYBRID   */
#include "http_get.h"
#include "icons.h"
#include "mainwindow.h"
#include "misc.h"
#include "misc_gtk.h"
#include "misc_strings.h"
#include "options.h"
#include "parse_message.h"
#include "send_command.h"
#include "status_page.h"
#include "statusbar.h"

extern GtkWidget             *downloads_view;
extern GtkListStore          *downloads_store;

gint                          downloads_num_paused = 0;


enum
{
 TOGGLE_SHOWHIDE_JUNK,
 TOGGLE_KEEPEXT,
 TOGGLE_VERIFYCANCELS,
 TOGGLE_NO_ACTIVE_CANCEL,
 TOGGLE_SAVE_CORRUPTED,
 TOGGLE_NO_IDS_PAUSE,
 TOGGLE_GREENPBPARTS,
 TOGGLE_SHOW_TRANS_AS_PERCENT,
 TOGGLE_BLACKLISTONDL,
 TOGGLE_UNSELECT_AFTER_ACTION,
 TOGGLE_SELMODE,
 TOGGLE_BOLD_FILENAMES,
} ;


/* functions */

static void       downloads_onPreviewSelected_preview_file (Download *dl);


/******************************************************************************
 *
 *     downloads_popup_unselect_all_if_wanted
 *
 ***/

static void
downloads_popup_unselect_all_if_wanted (void)
{
	if ( opt_get_bool(OPT_GUI_DOWNLOADS_UNSELECT_ALL_AFTER_ACTION) )
	{
		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(downloads_view));
		gtk_tree_selection_unselect_all(selection);
	}
}



/******************************************************************************
 *
 *   downloads_onPauseSelected
 *
 ***/

static void
downloads_onPauseSelected (GtkWidget *widget, gpointer data)
{
	Download    *dl;
	GSList      *selected, *node;

	selected = downloads_get_selected();

	for ( node = selected; node != NULL; node = node->next )
	{
		dl = (Download*)node->data;

		if ((dl) && (!dl->done) && dl->status != STATUS_PAUSED && dl->status != STATUS_PAUSING )
		{
			send_pause_download (dl->hash);
			dl->status = STATUS_PAUSING;
			gtk_list_store_set(downloads_store, &dl->iter, 0, dl, -1); /* force row to be re-drawn */
		}
	}

	g_slist_free(selected);

	send_get_space_stats();

	downloads_popup_unselect_all_if_wanted();
}


/******************************************************************************
 *
 *   downloads_onResumeSelected
 *
 ***/

static void
downloads_onResumeSelected (GtkWidget *widget, gpointer data)
{
	Download    *dl;
	GSList      *selected, *node;

	selected = downloads_get_selected();

	for ( node = selected;  node != NULL;  node = node->next)
	{
		dl = (Download*)node->data;

		if ((!dl) || dl->done || dl->status == STATUS_LOOKING || dl->status == STATUS_DOWNLOADING || dl->status == STATUS_RESUMING )
			continue;

		send_resume_download (dl->hash);
		dl->status = STATUS_RESUMING;
		gtk_list_store_set(downloads_store, &dl->iter, 0, dl, -1); /* force row to be re-drawn */
	}

	g_slist_free(selected);

	send_get_space_stats();

	downloads_popup_unselect_all_if_wanted();
}



/******************************************************************************
 *
 *   downloads_onPreviewSelected_preview_file
 *
 ***/

static void
downloads_onPreviewSelected_preview_file (Download *dl)
{
	gchar *preview_filename = NULL;

	if ( dl->status == STATUS_COMPLETE )
		preview_filename = g_strdup_printf ("%s/%s", option_incoming, dl->name);

	/* TODO: XXX - should it be option_incoming or option_temp here? */
	else if ( dl->status == STATUS_CORRUPTED )
	{
		gchar *basefn = g_path_get_basename(dl->previewname);
		if (basefn)
		{
			preview_filename = g_strdup_printf("%s/%s", option_incoming, preview_filename);
			g_free(basefn);
		}
	}
	else
		preview_filename = g_strdup(dl->previewname);

	if ( spawn_file_preview(preview_filename, dl->ext) == 0 )
	{
		G_FREE(preview_filename);
		return;
	}

	if ((coreversion%100) >= 48)
	{
		g_printerr (_("Could not preview '%s'!\n"), preview_filename);
		G_FREE(preview_filename);
		return;
	}

	/* PRE-V49 cores: old-style x.part filename did not work, let's try new-style x.1.part filename */
	if ( dl->status != STATUS_COMPLETE )
	{
		gchar *dot, *newstylefn;

		newstylefn = g_new0(gchar, strlen(preview_filename)+2);	/* + strlen (".1") */
		strcpy (newstylefn, preview_filename);	/* we know we have enough space */

		dot = strrchr (newstylefn, '.');
		g_return_if_fail ( dot != NULL);
		g_return_if_fail ( strcmp (dot, ".part") == 0 );

		strcpy (dot, ".1.part");	/* we know there is enough space */

		/* secondly, try new-style x.1.part filename */
		if (spawn_file_preview(newstylefn, dl->ext)!=0)
			g_printerr (_("Could not preview '%s' or '%s'!\n"), preview_filename, newstylefn);

		g_free(newstylefn);
	}
	else g_printerr (_("Could not preview '%s'!\n"), preview_filename);

	G_FREE(preview_filename);
}



/******************************************************************************
 *
 *   downloads_onPreviewSelected
 *
 ***/

static void
downloads_onPreviewSelected (GtkWidget *widget, gpointer data)
{
	GSList *selected, *node;

	selected = downloads_get_selected();

	for ( node = selected;  node != NULL;  node = node->next )
	{
		Download *dl = (Download*)node->data;

		if (dl)
		{
			if (dl->previewname)
				downloads_onPreviewSelected_preview_file (dl);
			else
				g_printerr (_("This core version does not supply us with the x.part filenames.\n"));
		}
	}

	g_slist_free(selected);

	downloads_popup_unselect_all_if_wanted();
}


/******************************************************************************
 *
 *   downloads_onWriteSelectedLinks
 *
 ***/

static void
downloads_onWriteSelectedLinks (GtkWidget *widget, gpointer data)
{
	Download *dl;
	GSList   *selected, *node;

	selected = downloads_get_selected();

	for (node = selected; node != NULL; node = node->next)
	{
		dl = (Download*)node->data;

		if (dl)
			status_msg_print_ed2klink (dl->name_utf8, dl->size, dl->hash);
	}

	g_slist_free(selected);

	downloads_popup_unselect_all_if_wanted();
}



/******************************************************************************
 *
 *   downloads_onResumeAll
 *
 ***/

static void
downloads_onResumeAll (GtkWidget *widget, gpointer data)
{
	GSList   *all, *node;

	all = downloads_get_all();

	for (node = all; node != NULL; node = node->next)
	{
		Download *dl = (Download*)node->data;

		if ((!dl) || dl->done || dl->status == STATUS_LOOKING || dl->status == STATUS_DOWNLOADING || dl->status == STATUS_RESUMING )
			continue;

		send_resume_download (dl->hash);
		dl->status = STATUS_RESUMING;
		gtk_list_store_set(downloads_store, &dl->iter, 0, dl, -1); /* force row to be re-drawn */
	}

	g_slist_free(all);

	send_get_space_stats();

	statusbar_extra_message ("%s", _(" All downloads resumed."));
}



/******************************************************************************
 *
 *   downloads_onPauseAll
 *
 ***/

void
downloads_onPauseAll (GtkWidget *widget, gpointer data)
{
	GSList   *all, *node;
	gboolean  opt_no_IDS_pause;
	guint     pmode = GPOINTER_TO_UINT(data);	/* pause mode */

	all = downloads_get_all();

	downloads_num_paused = 0;
	opt_no_IDS_pause = opt_get_bool(OPT_GUI_DOWNLOADS_NO_INSUFFICIENT_DISK_SPACE_PAUSE);

	for (node = all; node != NULL; node = node->next)
	{
		Download *dl     = (Download*)node->data;
		gboolean  pause  = FALSE;

		if ((!dl) || (dl->done))
			continue;

		switch (pmode)
		{
			case PAUSE_MODE_ALL:
				pause = TRUE;
				break;
			case PAUSE_MODE_ZEROSPEED:
				pause = (dl->speed == 0.0);
				break;
			case PAUSE_MODE_NOSOURCES:
				pause = (dl->sources == 0);
				break;
			case PAUSE_MODE_NOTFULLYAVAILABLE:
				pause = (dl->avail < 100);
				break;
		}

		if ( pmode != PAUSE_MODE_ALL  &&  dl->status == STATUS_IDS  &&  (opt_no_IDS_pause)  )
			pause = FALSE;

		if (!pause)
			continue;

		if ( dl->status == STATUS_PAUSING || dl->status == STATUS_PAUSED || dl->status == STATUS_COMPLETE )
			continue;

		downloads_num_paused++;

		send_pause_download (dl->hash);
		dl->status = STATUS_PAUSING;
		gtk_list_store_set(downloads_store, &dl->iter, 0, dl, -1); /* force row to be re-drawn */
	}

	g_slist_free(all);

	send_get_space_stats();

	statusbar_extra_message ("%s", _(" All downloads paused."));
}



/******************************************************************************
 *
 *   downloads_onClearCompleted
 *
 ***/

static void
downloads_onClearCompleted (GtkWidget *widget, gpointer data)
{
	GSList *all, *node;

	g_return_if_fail ( downloads_get_completed_items_count() > 0 );

	all = downloads_get_all();

	for ( node = all;  node != NULL;  node = node->next )
	{
		Download *dl = (Download*)node->data;

		if ((dl) && ( dl->status == STATUS_COMPLETE  ||  dl->status == STATUS_CORRUPTED ))
			download_remove(dl);
	}
}



/******************************************************************************
 *
 *   downloads_onGetJigleServerList
 *
 ***/

static void
downloads_onGetJigleServerList (GtkWidget *widget, gpointer data)
{
	Download  *dl;
	GSList    *selected, *node;
	gchar     *url;

	selected = downloads_get_selected();

	for ( node = selected; node != NULL; node = node->next )
	{
		dl = (Download*)node->data;
		if ((dl) && !dl->done)
		{
			url = g_strdup_printf ("http://jigle.com/search?p=ed2k:%u:%s&ma=1&d=1&a=0&l=10&t=7&x=&sl=1&su=&v=1",
		                       dl->size, hash_to_hash_str(dl->hash));
			if (url)
			{
				invoke_browser_with_url(url);
				http_get_retrieve_serverlist(url,0);
				g_free(url);
			}
		}
	}

	g_slist_free(selected);

	downloads_popup_unselect_all_if_wanted();
}


/******************************************************************************
 *
 *   downloads_onSetPriority
 *
 ***/

static void
downloads_onSetPriority (GtkWidget *widget, gpointer data)
{
	Download  *dl;
	GSList    *selected, *node, *all;
	guint      prio;

	selected = downloads_get_selected();
	prio = GPOINTER_TO_UINT(data);

	if ( prio == DL_PRIO_HIGHEST  &&  g_slist_length(selected) > 1 )
	{
		statusbar_extra_message(_("You can only set ONE download to 'highest' priority."));
		g_slist_free(selected);
		downloads_popup_unselect_all_if_wanted();
		return;
	}

	/* only one DL can have 'highest' priority, so we need to set the
	 * previous 'highest' back to 'high' priority */
	if ( prio == DL_PRIO_HIGHEST )
	{
		all = downloads_get_all();
		for ( node = all;  node != NULL;  node = node->next )
		{
			dl = (Download*)node->data;
			if (dl->priority == DL_PRIO_HIGHEST)
				dl->priority = DL_PRIO_HIGH;
		}
		g_slist_free(all);
	}

	for (node = selected; node != NULL; node = node->next)
	{
		dl = (Download*)node->data;
		if ((dl) && !dl->done)
		{
			send_set_download_priority (dl->hash, prio);
			dl->priority = prio;
			gtk_list_store_set(downloads_store, &dl->iter, 0, dl, -1); /* force row to be re-drawn */
		}
	}

	g_slist_free(selected);

	downloads_popup_unselect_all_if_wanted();
}



/******************************************************************************
 *
 *   downloads_onHideColumn
 *
 ***/

static void
downloads_onHideColumn (GtkWidget *widget, gpointer data)
{
	g_return_if_fail ( data != NULL );
	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(data), FALSE);
}



/******************************************************************************
 *
 *   downloads_onUnhideColumn
 *
 ***/

static void
downloads_onUnhideColumn (GtkWidget *widget, gpointer data)
{
	g_return_if_fail ( data != NULL );
	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(data), TRUE);
}


/******************************************************************************
 *
 *   downloads_onCancelSelected
 *
 ***/

static void
downloads_onCancelSelected (GtkWidget *widget, gpointer data)
{
	gboolean   blacklist = (GPOINTER_TO_UINT(data) == 1);
	gboolean   cancelerrdls = FALSE;  /* trying to cancel downloads with error?    */
	gboolean   cancelhashdls = FALSE; /* trying to cancel downloads while hashing? */
	GSList    *selected, *node, *toconfirm = NULL;
	GString   *names_to_confirm = NULL;

	selected = downloads_get_selected();

	for ( node = selected; node != NULL; node = node->next )
	{
		Download    *dl = (Download*)node->data;
		gboolean     need_confirmation;

		if (!dl)
			continue;

		if (dl->done || dl->status == STATUS_COMPLETE || dl->status == STATUS_CORRUPTED || dl->status == STATUS_COMPLETING)
			continue;

		if (dl->status == STATUS_HASHING || dl->status == STATUS_HASHING2)
		{
			cancelhashdls = TRUE;
			continue;
		}

		if (dl->status == STATUS_ERRLOADING || dl->status == STATUS_ERRHASHING)
		{
			cancelerrdls = TRUE;
			continue;
		}

		need_confirmation = opt_get_bool(OPT_GUI_DOWNLOADS_VERIFY_CANCEL);

		if (opt_get_bool(OPT_GUI_NO_ACTIVE_DOWNLOAD_CANCEL) && dl->status!=STATUS_PAUSED && dl->status!=STATUS_PAUSING)
			need_confirmation = TRUE;

		if (need_confirmation)
		{
			if (!names_to_confirm)
				names_to_confirm = g_string_new("\n");

			names_to_confirm = g_string_append(names_to_confirm, dl->name);
			names_to_confirm = g_string_append_c(names_to_confirm, '\n');

			toconfirm = g_slist_append (toconfirm, dl);
		}
		else
		{
			send_cancel_download (dl->hash);

			dl->cancelled = TRUE;

			if (blacklist)
				add_to_blacklist (dl->hash, dl->size, dl->name);
		}
	}


	/* now show confirm dialog for those downloads
	 * for which we need confirmation to cancel them */

	if ( g_slist_length(toconfirm) > 0 )
	{
		GtkWidget *dialog;
		gchar     *utf8a, *utf8b;
		gint       ret;

		utf8a = TO_UTF8(_("Are you sure you want to cancel the following downloads?"));
		utf8b = TO_UTF8(_("The unfinished files will be deleted from your harddrive."));

		/* filenames ARE already in UTF8 - we don't want to convert them twice (thanks to EL) */
		dialog = gtk_message_dialog_new ( GTK_WINDOW(window),
		                                  GTK_DIALOG_DESTROY_WITH_PARENT,
		                                  GTK_MESSAGE_QUESTION,
		                                  GTK_BUTTONS_YES_NO,
		                                  "%s\n%s\n%s\n\n", utf8a, names_to_confirm->str, utf8b);

		ret = gtk_dialog_run (GTK_DIALOG(dialog));

		gtk_widget_destroy(dialog);

		if ( ret == GTK_RESPONSE_YES )
		{
			for (node = toconfirm; node != NULL; node = node->next)
			{
				Download *dl = (Download*)node->data;

				send_cancel_download (dl->hash);
				dl->cancelled = TRUE;

				if (blacklist)
					add_to_blacklist (dl->hash, dl->size, dl->name);
			}
		}

		FREE_UTF8(utf8a);
		FREE_UTF8(utf8b);
	} /* end if g_slist_length(toconfirm) > 0 */

	if (cancelhashdls)
	{
		misc_gtk_ok_dialog (GTK_MESSAGE_WARNING,
			_("You can't cancel downloads while they are hashing. Sorry. Please try again later."));
	}

	if (cancelerrdls)
	{
		misc_gtk_ok_dialog (GTK_MESSAGE_WARNING,
			_("One or more downloads selected have either had\nan error during loading or\n"
			  "an error during hashing.\n\nYou can't just cancel these downloads.\n\n"
			  "You will have to remove them manually\n(when the donkey is not running).\n\n"
			  "The right-click pop-up menu\nshould show you which x.part file\na certain download is.\n") );
	}

	g_slist_free(selected);
	g_slist_free(toconfirm);

	if (names_to_confirm)
		g_string_free(names_to_confirm, TRUE);

	downloads_popup_unselect_all_if_wanted();
}


/******************************************************************************
 *
 *   downloads_onOptionToggled
 *
 ***/

static void
downloads_onOptionToggled (GtkWidget *widget, gpointer data)
{
	guint optnum = GPOINTER_TO_UINT(data);

	switch (optnum)
	{
		case TOGGLE_SHOWHIDE_JUNK:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_HIDE_JUNK);
			downloads_update_view_appearance();
		break;

		case TOGGLE_VERIFYCANCELS:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_VERIFY_CANCEL);
		break;

		case TOGGLE_KEEPEXT:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_KEEP_EXTENSION_WHEN_HIDING_JUNK);
			downloads_update_view_appearance();
		break;

		case TOGGLE_SAVE_CORRUPTED:
			option_save_corrupted ^= 1;
			send_save_corrupted (option_save_corrupted);
		break;

		case TOGGLE_NO_IDS_PAUSE:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_NO_INSUFFICIENT_DISK_SPACE_PAUSE);
		break;

		case TOGGLE_NO_ACTIVE_CANCEL:
			opt_toggle_bool(OPT_GUI_NO_ACTIVE_DOWNLOAD_CANCEL);
		break;

		case TOGGLE_GREENPBPARTS:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_BLUE_PROGRESSBAR);
		break;

		case TOGGLE_SHOW_TRANS_AS_PERCENT:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_SHOW_TRANSFERED_IN_PERCENT);
			downloads_update_view_appearance();
		break;

		case TOGGLE_BLACKLISTONDL:
			opt_toggle_bool(OPT_GUI_DOWNLOADS_BLACKLIST_ON_DOWNLOAD);
		break;

		case TOGGLE_SELMODE:
		{
			GtkTreeSelection *selection;

			opt_toggle_bool(OPT_GUI_DOWNLOADS_SINGLE_SELECTION_MODE);

			/* set multi- or single- selection mode */
			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(downloads_view));

			if ( opt_get_bool(OPT_GUI_DOWNLOADS_SINGLE_SELECTION_MODE) )
			{
				gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
			}
			else
			{
				gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
			}
		}
		break;

		case TOGGLE_UNSELECT_AFTER_ACTION:
			opt_toggle_bool(OPT_GUI_SERVERS_UNSELECT_ALL_AFTER_ACTION);
		break;

		case TOGGLE_BOLD_FILENAMES:
		{
			opt_toggle_bool(OPT_GUI_DOWNLOADS_FILENAMES_BOLD);
			downloads_update_view_appearance();
		}
		break;

		default:
			g_printerr ("GUI: Fix me - called with unknown value in %s\n)", __FUNCTION__);
		break;
	}
}


/******************************************************************************
 *
 *   downloads_popup_menu_append_info_string
 *
 ***/

static void
downloads_popup_menu_append_info_string (GtkWidget *menu)
{
	const gchar  *utf8 = NULL;
	GSList       *selected;
	Download     *dl;

	selected = downloads_get_selected();

	if (g_slist_length(selected) != 1)
	{
		g_slist_free(selected);
		return;
	}

	dl = (Download*) selected->data;
	g_return_if_fail ( dl != NULL );

	if (!dl->previewname)
	{
		g_slist_free (selected);
		return;
	}

	if ( dl->status != STATUS_COMPLETE && dl->status != STATUS_CORRUPTED )
	{
		if ((coreversion%100) < 49) /* old cores */
		{
			gchar *basename = (dl->previewname) ? g_path_get_basename(dl->previewname) : NULL;
			if ( dl->transfered <= dl->size )
			{
				gchar *leftstr = g_strdup_printf(_(" %s left"), (gchar*)human_size(dl->size - dl->transfered));
				utf8 = UTF8_SHORT_PRINTF ("%s - %s", (basename) ? basename : "???", leftstr);
				G_FREE(leftstr);
			}
			else
				utf8 = UTF8_SHORT_PRINTF ("%s - %s", (basename) ? basename : "???", _(" not much left"));

			G_FREE(basename);
		}
		else /* core >=v49 */
		{
			if ( dl->transfered <= dl->size )
				utf8 = UTF8_SHORT_PRINTF(_(" %s left"), (gchar*)human_size(dl->size - dl->transfered));
			else
				utf8 = UTF8_SHORT_PRINTF ("%s", _(" not much left"));
		}
	}
	else
		utf8 = UTF8_SHORT_PRINTF("%s", _("Finished."));

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_menu_item (menu, utf8, NULL, NULL, ICON_MENU_STATUS, TRUE );

	g_slist_free (selected);

}


/******************************************************************************
 *
 *   downloads_popup_menu_options
 *
 ***/

static GtkWidget *
downloads_popup_menu_options (GdkEventButton *event)
{
	GtkWidget   *menu = NULL;
	GList       *columns, *node;

	IconName     icon_yes = ICON_OPTION_TRUE;
	IconName     icon_no  = ICON_OPTION_FALSE;

	menu  = gtk_menu_new();

	misc_gtk_add_menu_item (menu, _(" hide junk in filenames shown "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_SHOWHIDE_JUNK),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_HIDE_JUNK)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" keep file extensions when removing junk in download list "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_KEEPEXT),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_KEEP_EXTENSION_WHEN_HIDING_JUNK)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_menu_item (menu, _(" verify download cancels "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_VERIFYCANCELS),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_VERIFY_CANCEL)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" never cancel non-paused downloads without confirmation "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_NO_ACTIVE_CANCEL),
	                        (opt_get_bool(OPT_GUI_NO_ACTIVE_DOWNLOAD_CANCEL)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" keep/save corrupted downloads instead of deleting them (core) "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_SAVE_CORRUPTED), (option_save_corrupted==1) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" skip files with Insuff.Disk Space on conditional 'pause all' "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_NO_IDS_PAUSE),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_NO_INSUFFICIENT_DISK_SPACE_PAUSE)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" progress bars: use green shading for available parts "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_GREENPBPARTS),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_BLUE_PROGRESSBAR)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" show filenames in bold print "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_BOLD_FILENAMES),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_FILENAMES_BOLD)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" show 'transfered' in percent "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_SHOW_TRANS_AS_PERCENT),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_SHOW_TRANSFERED_IN_PERCENT)) ? icon_yes : icon_no, TRUE);


	misc_gtk_add_menu_item (menu, _(" add new downloads automatically to blacklist "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_BLACKLISTONDL),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_BLACKLIST_ON_DOWNLOAD)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_menu_item (menu, _(" unselect all items after each action "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_UNSELECT_AFTER_ACTION),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_UNSELECT_ALL_AFTER_ACTION)) ? icon_yes : icon_no, TRUE);

	misc_gtk_add_menu_item (menu, _(" use single selection mode in download list "), downloads_onOptionToggled,
	                        GUINT_TO_POINTER(TOGGLE_SELMODE),
	                        (opt_get_bool(OPT_GUI_DOWNLOADS_SINGLE_SELECTION_MODE)) ? icon_yes : icon_no, TRUE);

	/* 'hide column' item */

	if (event)
	{
		GtkTreeViewColumn  *col = NULL;

		(void) gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(downloads_view), (gint)event->x, (gint)event->y, NULL, &col, NULL, NULL);

		/* if there are no download entries at all, we get col == NULL, thanks to Daniel for finding this */
		if (col)
		{
			const gchar *utf8 = UTF8_SHORT_PRINTF(_(" hide column '%s' "), gtk_tree_view_column_get_title(col));

			misc_gtk_add_menu_separator (menu);

			misc_gtk_add_menu_item (menu, utf8, downloads_onHideColumn, (gpointer) col, ICON_MENU_HIDE_COLUMN, TRUE);
		}
	}

	
	/* 'unhide column X' items */

	columns = gtk_tree_view_get_columns (GTK_TREE_VIEW(downloads_view));

	if ( g_list_length(columns) > 0 )
		misc_gtk_add_menu_separator (menu);

	for ( node = columns; node != NULL; node = node->next )
	{
		GtkTreeViewColumn *col;
		const gchar       *utf8;

		if (!node->data)
			continue;

		col = GTK_TREE_VIEW_COLUMN(node->data);

		if (gtk_tree_view_column_get_visible(col))
			continue;

		utf8 = UTF8_SHORT_PRINTF(_(" show hidden column '%s' "), gtk_tree_view_column_get_title(col));

		misc_gtk_add_menu_item (menu, utf8, downloads_onUnhideColumn, (gpointer) col, ICON_MENU_UNHIDE_COLUMN, TRUE);
	}
	g_list_free(columns);

	return menu;
}





/******************************************************************************
 *
 *   downloads_popup_menu
 *
 ***/

void
downloads_popup_menu (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	GtkTreeSelection  *selection;
	GtkWidget         *menu, *pmenu, *cmenu, *pausemenu, *omenu;
	gint               sel_count;

	g_return_if_fail ( downloads_view != NULL );

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(downloads_view));
	sel_count = misc_gtk_tree_selection_count_selected_rows (selection);

	/* priority sub-menu */
	pmenu  = gtk_menu_new();

	/* do not confuse with 'old' v5x versions before versioning scheme change */
	if (coreversion >= 49 && core_build_date >  0x030601 )
		misc_gtk_add_menu_item (pmenu, _(" highest ")  , downloads_onSetPriority, GUINT_TO_POINTER(DL_PRIO_HIGHEST), ICON_MENU_PRIO_HIGH  , TRUE );

	misc_gtk_add_menu_item (pmenu, _(" high ")  , downloads_onSetPriority, GUINT_TO_POINTER(DL_PRIO_HIGH), ICON_MENU_PRIO_HIGH  , TRUE );
	misc_gtk_add_menu_item (pmenu, _(" normal "), downloads_onSetPriority, GUINT_TO_POINTER(DL_PRIO_NORMAL), ICON_MENU_PRIO_MEDIUM, TRUE );
	misc_gtk_add_menu_item (pmenu, _(" low ")   , downloads_onSetPriority, GUINT_TO_POINTER(DL_PRIO_LOW), ICON_MENU_PRIO_LOW   , TRUE );


	/* cancel sub-menu */
	cmenu = gtk_menu_new();
	misc_gtk_add_menu_item (cmenu, _(" cancel selected downloads ")  , downloads_onCancelSelected, GUINT_TO_POINTER(0), ICON_MENU_DL_CANCEL, TRUE);
	misc_gtk_add_menu_item (cmenu, _(" cancel and add to blacklist "), downloads_onCancelSelected, GUINT_TO_POINTER(1), ICON_MENU_BLACKLIST, TRUE);

	/* pause all... sub-menu */
	pausemenu = gtk_menu_new();
	misc_gtk_add_menu_item (pausemenu, _(" pause all idle downloads ")               , downloads_onPauseAll, GUINT_TO_POINTER(PAUSE_MODE_ZEROSPEED)        , ICON_MENU_PAUSE_ALL, TRUE);
	misc_gtk_add_menu_item (pausemenu, _(" pause all downloads w/o sources ")        , downloads_onPauseAll, GUINT_TO_POINTER(PAUSE_MODE_NOSOURCES)        , ICON_MENU_PAUSE_ALL, TRUE);
	misc_gtk_add_menu_item (pausemenu, _(" pause all not fully available downloads "), downloads_onPauseAll, GUINT_TO_POINTER(PAUSE_MODE_NOTFULLYAVAILABLE), ICON_MENU_PAUSE_ALL, TRUE);
	misc_gtk_add_menu_item (pausemenu, _(" pause all downloads ")                    , downloads_onPauseAll, GUINT_TO_POINTER(PAUSE_MODE_ALL)              , ICON_MENU_PAUSE_ALL, TRUE);

	/* main pop-up menu */
	menu  = gtk_menu_new();

	misc_gtk_add_menu_item (menu, _(" pause ")  , downloads_onPauseSelected  , NULL, ICON_MENU_PAUSE  , ( sel_count > 0 ) );
	misc_gtk_add_menu_item (menu, _(" resume  "), downloads_onResumeSelected , NULL, ICON_MENU_RESUME , ( sel_count > 0 ) );
	misc_gtk_add_menu_item (menu, _(" preview "), downloads_onPreviewSelected, NULL, ICON_MENU_PREVIEW, ((sel_count > 0) && ( CORE_IS_LOCAL == TRUE )) );

	misc_gtk_add_submenu   (menu, _(" set priority "), ICON_MENU_PRIO_MEDIUM, pmenu, ( sel_count > 0 ) );
	misc_gtk_add_submenu   (menu, _(" cancel ")      , ICON_MENU_DL_CANCEL  , cmenu, ( sel_count > 0 ) );

	misc_gtk_add_menu_item (menu, _(" write ed2k-link to log/clipboard "), downloads_onWriteSelectedLinks, NULL, ICON_MENU_STATUS, ( sel_count > 0 ) );

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_submenu   (menu, _(" pause all ")   , ICON_MENU_PAUSE_ALL  , pausemenu, TRUE );

	misc_gtk_add_menu_item (menu, _(" resume all "), downloads_onResumeAll, NULL, ICON_MENU_RESUME_ALL, TRUE);

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_menu_item (menu, _(" clear completed downloads "), downloads_onClearCompleted, NULL, ICON_MENU_CANCEL, ( downloads_get_completed_items_count() > 0 ) );

	misc_gtk_add_menu_separator (menu);

	misc_gtk_add_menu_item (menu, _(" import and show jigle serverlist  "), downloads_onGetJigleServerList, NULL, ICON_MENU_SEARCH,
	                         (( sel_count > 0 ) && (IS_OVERNET == FALSE || IS_HYBRID == TRUE)) );

	omenu = downloads_popup_menu_options(event);

	misc_gtk_add_submenu   (menu, _(" downloads options ") , ICON_MENU_OPTIONS , omenu, TRUE );


	downloads_popup_menu_append_info_string(menu);

	gtk_widget_show(menu);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, (event) ? event->button : 0, gdk_event_get_time((GdkEvent*)event));
}



/******************************************************************************
 *
 *     downloads_onKeyPress
 *
 ***/

gint
downloads_onKeyPress  (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	guint key = gdk_keyval_to_lower(event->keyval);

	switch (key)
	{
		case GDK_Delete:
					downloads_onCancelSelected(NULL,NULL);
					break;
		case 'p':		downloads_onPauseSelected(NULL, NULL);
					break;
		case 'r':		downloads_onResumeSelected(NULL, NULL);
					break;
		case 'l':		downloads_onSetPriority(NULL, GUINT_TO_POINTER(DL_PRIO_LOW));
					break;
		case 'n':		downloads_onSetPriority(NULL, GUINT_TO_POINTER(DL_PRIO_NORMAL));
					break;
		case 'h':		downloads_onSetPriority(NULL, GUINT_TO_POINTER(DL_PRIO_HIGH));
					break;
		case 'u':
		{
			GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(downloads_view));
			gtk_tree_selection_unselect_all(selection);
		}
		break;
		case 'a':
		{
			GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(downloads_view));
			gtk_tree_selection_select_all(selection);
		}
		break;
		default:		return FALSE;
	}
	return TRUE; /* has been dealt with - if key is not in list we return false right away */
}



