/***************************************************************************
                          progressbar.c  -  description
                             -------------------
    begin                : Mon Feb 10 2003
    copyright            : (C) 2003 by Tim-Philipp Mller
    email                : t.i.m@orange.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "colors.h"
#include "options.h"
#include "mainwindow.h"
#include "progressbar.h"

#define PROGRESSBAR_MINIMUM_WIDTH     50
#define PROGRESSBAR_INVISIBLE_WIDTH   25
#define PROGRESSBAR_MINIMUM_HEIGHT    5

#define GAPSTATUS_ASKED              1000
#define GAPSTATUS_FILLED             2000

enum
{
 PROGRESSBAR_COL_SCHEME_GREEN = 0,
 PROGRESSBAR_COL_SCHEME_BLUE,
 PROGRESSBAR_COL_SCHEME_N
} ;

enum
{
 PROGRESSBAR_PEN_PAUSED = 0,
 PROGRESSBAR_PEN_GOT_IT,
 PROGRESSBAR_PEN_NOT_AVAILABLE,
 PROGRESSBAR_PEN_ASKED_FOR,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_128,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_64,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_32,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_16,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_8,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_4,
 PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_2,
 PROGRESSBAR_PEN_AVAILABLE,
 PROGRESSBAR_PENS_N
} ;

GdkColor  progressbar_pen_colours[PROGRESSBAR_PENS_N];
GdkGC    *progressbar_pens[PROGRESSBAR_PENS_N] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };

GdkColor  somecol = { 0, 0xb400, 0x0000, 0x0000 };

static void
progressbar_set_pen_colour (guint32 rgb, guint colnum)
{
	progressbar_pen_colours[colnum].pixel = 0;
	progressbar_pen_colours[colnum].red   = (rgb&0xff0000) >> 8;
	progressbar_pen_colours[colnum].green = (rgb&0x00ff00) >> 0;
	progressbar_pen_colours[colnum].blue  = (rgb&0x0000ff) << 8;

	if (!gdk_colormap_alloc_color (gdk_colormap_get_system(), &progressbar_pen_colours[colnum], FALSE, TRUE))
		g_printerr("Color allocation in %s failed.\n", __FUNCTION__);
}


/******************************************************************************
 *
 *   progressbar_select_colour_theme
 *
 *   Fills the progressbar_pen_colours array with
 *    the right colours for each scheme.
 *
 ***/

static void
progressbar_select_colour_theme (guint theme)
{
	switch ( theme )
	{
		case PROGRESSBAR_COL_SCHEME_GREEN:
		{
			/* remember these: darker: 0x06500C --- even darker: 0x05440B */
			progressbar_set_pen_colour (0xE0E0E0, PROGRESSBAR_PEN_PAUSED);
			progressbar_set_pen_colour (0x000000, PROGRESSBAR_PEN_GOT_IT);
			progressbar_set_pen_colour (0xB40000, PROGRESSBAR_PEN_NOT_AVAILABLE);
			progressbar_set_pen_colour (0x086510, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_128);
			progressbar_set_pen_colour (0x096E11, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_64);
			progressbar_set_pen_colour (0x0A7D13, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_32);
			progressbar_set_pen_colour (0x0B8F16, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_16);
			progressbar_set_pen_colour (0x0C9E19, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_8);
			progressbar_set_pen_colour (0x0Eb61C, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_4);
			progressbar_set_pen_colour (0x10C91F, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_2);
			progressbar_set_pen_colour (0x11D922, PROGRESSBAR_PEN_AVAILABLE);
			progressbar_set_pen_colour (0xA3A3A3, PROGRESSBAR_PEN_ASKED_FOR);
			break;
		}

		case PROGRESSBAR_COL_SCHEME_BLUE:
		{
			/* remember these: darker: 0x5081AC --- even darker: 0x404894 */
			progressbar_set_pen_colour (0xE0E0E0, PROGRESSBAR_PEN_PAUSED);
			progressbar_set_pen_colour (0x000000, PROGRESSBAR_PEN_GOT_IT);
			progressbar_set_pen_colour (0xB40000, PROGRESSBAR_PEN_NOT_AVAILABLE);
			progressbar_set_pen_colour (0x598FBF, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_128);
			progressbar_set_pen_colour (0x5D95C7, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_64);
			progressbar_set_pen_colour (0x619CD0, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_32);
			progressbar_set_pen_colour (0x72BAD7, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_16);
			progressbar_set_pen_colour (0x76C0DE, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_8);
			progressbar_set_pen_colour (0x7AC7E6, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_4);
			progressbar_set_pen_colour (0x7FCFEF, PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_2);
			progressbar_set_pen_colour (0x87DDFF, PROGRESSBAR_PEN_AVAILABLE);
			progressbar_set_pen_colour (0xA3A3A3, PROGRESSBAR_PEN_ASKED_FOR);
			break;
		}

		default:
		{
			g_return_if_reached();
		}
	}
}

/******************************************************************************
 *
 *   progressbar_init_brushes
 *
 ***/

void
progressbar_init_brushes (GdkPixmap *pixmap)
{
	guint i;

	g_return_if_fail ( pixmap != NULL );

	progressbar_select_colour_theme ( (opt_get_bool(OPT_GUI_DOWNLOADS_BLUE_PROGRESSBAR)) ? PROGRESSBAR_COL_SCHEME_GREEN : PROGRESSBAR_COL_SCHEME_BLUE );

	for (i=0; i < PROGRESSBAR_PENS_N; i++)
	{
		GdkGC *newgc;

		if (progressbar_pens[i])
			g_object_unref(G_OBJECT(progressbar_pens[i]));

		newgc = gdk_gc_new(pixmap);
		progressbar_pens[i] = newgc;
		gdk_gc_set_foreground(newgc, &progressbar_pen_colours[i]);
	}

}


/******************************************************************************
 *
 *   progressbar_sort_gaps
 *
 *   puts gaps in the right order
 *
 ***/

void
progressbar_sort_gaps (progressbarGap *pbgaps)
{
	guint     i, j, num;
	gboolean  need_sorting = FALSE;

	g_return_if_fail ( pbgaps != NULL );

	/* count number of gaps and check whether the gaps need sorting in the first place */
	for ( num = 0;  pbgaps[num].start!=0 || pbgaps[num].end!=0 || pbgaps[num].status!=0;  num++ )
	{
		if ( num > 0  &&  pbgaps[num].start < pbgaps[num-1].start )
				need_sorting = TRUE;
	}

	if ( num == 1  || !need_sorting )
		return;

	/* bubble sort */
	for (i=0; i < num-1; i++)
	{
		for (j=i+1; j < num; j++)
		{
			if (pbgaps[j].start < pbgaps[i].start)
			{
				progressbarGap swap = pbgaps[j];
				pbgaps[j] = pbgaps[i];
				pbgaps[i] = swap;
			}
		}
	}
}






/******************************************************************************
 *
 *   progressbar_get_shaded_gap_available_brush
 *
 *   returns a shade of green or whatever, depending
 *   on the number of sources we have for this gap
 *
 ***/

static GdkGC *
progressbar_get_shaded_gap_available_brush (guint sources)
{
	if (sources >= 128)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_128];

	if (sources >= 64)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_64];

	if (sources >= 32)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_32];

	if (sources >= 16)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_16];

	if (sources >= 8)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_8];

	if (sources >= 4)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_4];

	if (sources >= 2)
		return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE_MORE_THAN_2];

	return progressbar_pens[PROGRESSBAR_PEN_AVAILABLE];
}


/******************************************************************************
 *
 *   progressbar_get_brush_from_value
 *
 *   takes the gap status and returns the right colour/brush
 *
 ***/

G_INLINE_FUNC GdkGC *
progressbar_get_brush_from_value ( guint status, gboolean paused )
{

	/* got it? */
	if ( status == GAPSTATUS_FILLED )
		return progressbar_pens[PROGRESSBAR_PEN_GOT_IT];

	/* !status means 'not available' */
	if ( ((!status) && (paused)) || ((paused) && core_build_date < 0x03070B )) /* >= v49.5 */
		return progressbar_pens[PROGRESSBAR_PEN_PAUSED];

	/* not available? */
	if (!status)
		return progressbar_pens[PROGRESSBAR_PEN_NOT_AVAILABLE];

	/* asked for? */
	if ( (status-(status % GAPSTATUS_ASKED)) == GAPSTATUS_ASKED )
			return progressbar_pens[PROGRESSBAR_PEN_ASKED_FOR];

	/* this is a core fsck-up - don't show error message any more */
	if (status >= GAPSTATUS_ASKED)
	{
		if ( core_build_date < 0x03070B ) /* >= v49.5 */
			return progressbar_pens[PROGRESSBAR_PEN_ASKED_FOR];
		else
			return progressbar_pens[PROGRESSBAR_PEN_GOT_IT];
	}

	if (paused)
		return progressbar_pens[PROGRESSBAR_PEN_PAUSED];

	/* means NOW (after previous checks): available */
	return progressbar_get_shaded_gap_available_brush ( status );
}


/******************************************************************************
 *
 *   progressbar_make_pixbuf_from_gaplist
 *
 ***/

GdkPixbuf *
progressbar_make_pixbuf_from_gaplist (const progressbarGap *pbgaps, guint width, guint height, guint filesize, gboolean paused)
{
	GdkPixmap         *pixmap;
	GdkPixbuf         *pixbuf;
	guint              gapnum, pixel_correct;
	GdkGC             *brush = NULL;
	gfloat             w, gap_width_in_pixels, fwidth;
	gfloat             ffilesize = (gfloat) filesize;
	static gboolean    brushes_not_initialised = TRUE;
	static guint       last_theme_option = G_MAXUINT;

	g_return_val_if_fail ( filesize > 0, NULL );
	g_return_val_if_fail ( width    < 100000, NULL );

	if (!window)
		return NULL;

	/* too small? assume they don't want a progress bar */
	if ( width <= PROGRESSBAR_INVISIBLE_WIDTH )
		return NULL;

	if ( width < PROGRESSBAR_MINIMUM_WIDTH )
		width = PROGRESSBAR_MINIMUM_WIDTH;

	if ( height < PROGRESSBAR_MINIMUM_HEIGHT )
		height = PROGRESSBAR_MINIMUM_HEIGHT;

	pixmap = gdk_pixmap_new ( window->window, width, height, -1 );

	if ( (brushes_not_initialised) || last_theme_option != opt_get_bool(OPT_GUI_DOWNLOADS_BLUE_PROGRESSBAR) )
	{
		progressbar_init_brushes(pixmap);
		brushes_not_initialised = FALSE;
		last_theme_option = opt_get_bool(OPT_GUI_DOWNLOADS_BLUE_PROGRESSBAR);
	}

	gapnum = 0;
	pixel_correct = 0;

//	gdk_draw_rectangle ( pixmap, penblue, TRUE, 0, 0, width, height );

	w = 0.0;
	fwidth = (gfloat) width;
	while ( w < width )
	{
		gfloat        start = (gfloat) pbgaps[gapnum].start;
		gfloat        end = (gfloat) pbgaps[gapnum].end;
		guint         gapstatus = pbgaps[gapnum].status;

		if (start==0.0 && end==0.0 && gapstatus==0)
			break;

		if ( end < start )
		{
			guint swap = end; end = start; start = swap;
			g_print ("GUI DEBUG: oops, core sent gap with end < start !?\n");
		}

		gap_width_in_pixels = ((end-start)/ffilesize)*fwidth;

/*
		if ( gap_width_in_pixels > 5 )
		{
			gap_width_in_pixels = gap_width_in_pixels - pixel_correct;
			pixel_correct = 0;
		}
*/
		if ( gap_width_in_pixels <= 0 )
		{
			//pixel_correct = 0 - gap_width_in_pixels;
			gap_width_in_pixels = 1;
		}

		brush = progressbar_get_brush_from_value ( gapstatus, paused );

		gdk_draw_rectangle ( pixmap, brush, TRUE, w, 0, (guint)(w+gap_width_in_pixels)+2, height );

		gapnum++;

		while ( ((guint)(w + gap_width_in_pixels)) == (guint)w )
			gap_width_in_pixels += 0.2;

		w = w + gap_width_in_pixels;
	}

//	if (brush)
//		gdk_draw_rectangle ( pixmap, penblue, TRUE, w, 0, width-w, height );

	pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, NULL, 0, 0, 0, 0, width, height);
	g_return_val_if_fail (pixbuf != NULL, pixbuf);

	g_object_unref(G_OBJECT(pixmap));

	return pixbuf;
}


/******************************************************************************
 *
 *   progressbar_set_transfered_from_gaplist
 *
 ***/

void
progressbar_set_transfered_from_gaplist (progressbarGap *pbgaps, guint32 *trans)
{
	gint    i = 0;
	gulong  sum = 0;

	g_return_if_fail ( pbgaps != NULL );
	g_return_if_fail ( trans  != NULL );

	while (pbgaps[i].start!=0 || pbgaps[i].end!=0 || pbgaps[i].status!=0)
	{
		/* >= 0x04070B means >= v49.5 */
		if ( pbgaps[i].status == GAPSTATUS_FILLED ||
		       (pbgaps[i].status >= GAPSTATUS_ASKED && core_build_date >= 0x03070B) )
		{
			gint gaplength = pbgaps[i].end - pbgaps[i].start;

			if ( gaplength > 0 )
				sum = sum + gaplength;
		}
		i++;
	}

	*trans = (guint32) sum;

}


/******************************************************************************
 *
 *   progressbar_find_min_num_of_sources_from_gaplist
 *
 *   Checks the gaplist and returns the minimum number of sources
 *   (ie. the max. num of sources having one and the same block of
 *    a file available)
 *
 *   Returns min. number of sources or 0 if no gaplist or error

 ***/

guint
progressbar_find_min_num_of_sources_from_gaplist (progressbarGap *pbgaps)
{
	gint i       = 0;
  gint min_num = 0;

	g_return_val_if_fail ( pbgaps != NULL, 0 );

	for ( i = 0; (pbgaps[i].start!=0 || pbgaps[i].end!=0 || pbgaps[i].status!=0); i++ )
	{
		guint16 val;

		val = pbgaps[i].status;

		/* not available? */
		if ( val == 0 )
			continue;

		/* got it or asked for? */
		if ( val >= GAPSTATUS_ASKED )
			continue;

		/* means _now_ (after previous checks): available							*/
		min_num = MAX (min_num, val);
	}

	return min_num;

}




