/*
*   send_command.h
*
*   (c) 2001 Tim-Philipp Mueller <t.i.m@orange.net>
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ed2k_gui_send_command_h_included
#define ed2k_gui_send_command_h_included

#include "servers.h"

#include <glib.h>


// login / logout

void	send_login (const gchar *user, const gchar *pass);
void	send_logout (void);

// search

void	send_search (const gchar *str);
void	send_search_user (const gchar *str);
void	send_type_search (const gchar *str, const gchar *type, guint32 i_minbitrate);
void	send_extended_search (void);
void	send_extended_search_user (void);
void	send_more_search (void);		// get the next 200 results

void	send_search_with_extra_options (const gchar *str, guint32 i_minsize, guint32 i_maxsize, guint32 i_minavail);
void	send_type_search_with_extra_options (const gchar *str, const gchar *type, guint32 i_minsize, guint32 i_maxsize, guint32 i_minavail, guint32 i_minbitrate);

// downloads

void	send_download (const guint8 *hash);
void	send_new_download (const guint8 *hash, guint32 size, const gchar *fn);
void	send_start_sending_dl_status (void);
void	send_stop_sending_dl_status (void);
void	send_set_download_priority (const guint8 *hash, guint8 priority);
void	send_pause_download (const guint8 *hash);
void	send_resume_download (const guint8 *hash);
void	send_cancel_download (const guint8 *hash);
void	send_get_download_gaps (const guint8 *hash);

// uploads
void	send_start_sending_ul_status (void);
void	send_stop_sending_ul_status (void);

// servers

void	send_connect_to_server (guint32 ip, guint16 port);
void	send_disconnect (void);
void	send_get_serverlist (void);
void	send_add_server (guint32 ip, guint16 port);
void	send_remove_server (guint32 ip, guint16 port);
void	send_set_server_priority (guint32 ip, guint16 port, ServerPriority priority);

// shared stuff

void	send_get_shared_dirs (void);
void	send_get_shared_files(void);
void	send_unshare_dir_no (guint dir_no);
void	send_add_shared_dir (const gchar *dir);
void	send_add_shared_dir_structure (const gchar *dir);

// other
void	send_command (const gchar *cmd);
//void	send_big_dummy_packet (void);	// for testing/debugging - putting strain on the socket

// options

void	send_get_options (void);
void	send_set_max_speeds (guint dlmax, guint ulmax);
void	send_auto_connect (gboolean autoconnectON);
void	send_verbose (gboolean verboseON);
void	send_save_corrupted (gboolean savecorruptedON);
void	send_auto_server_remove (gboolean autoserverremoveON);
void	send_set_nickname (const gchar *nick);
void	send_set_incoming (const gchar *dir);
void	send_set_temp (const gchar *dir);
void	send_set_port (guint16 port);
void	send_set_admin_port (guint16 aport);
void	send_set_max_connections (guint maxcon);

void	send_get_peer_stats (void);
void	send_get_space_stats (void);
void	send_get_client_stats (void);

void    send_set_upload_priority (const guint8 *hash, guint newprio);

void    send_get_overnet_contacts (void);


#endif
