/* shared_page.c
*
*  populates shared page in notepad and deals with its functions
*
*  (c) 2001 Tim-Philipp Muller <t.i.m@orange.net>
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "colors.h"
#include "global.h"
#include "misc_strings.h"
#include "options.h"
#include "shared_dirs.h"
#include "shared_files.h"
#include "shared_page.h"

/* variables */

static GtkWidget	*spanedwin = NULL;
static GtkWidget	*shared_page_stats_label = NULL;

/******************************************************************************
 *
 *   shared_page_get_panedwin_separator_position
 *
 */

gint
shared_page_get_panedwin_separator_position(void)
{
	GtkHPaned	*panedwindow;

	g_return_val_if_fail (spanedwin != NULL, 0);

	panedwindow = GTK_HPANED(spanedwin);
	return (panedwindow->paned).child1_size;
}


/******************************************************************************
 *
 *   shared_page_set_stats_label_message
 *
 */

void
shared_page_set_stats_label_message (const gchar *txt)
{
	gchar *txt2;

	g_return_if_fail (txt!=NULL);
	g_return_if_fail (shared_page_stats_label!=NULL);

	txt2 = g_strdup (txt);
	g_return_if_fail (txt2!=NULL);

	/* replace all newline characters with whitespaces */
	g_strdelimit(txt2, "\n", ' ');

	/* shows if shared files are being hashed or not */
	gtk_label_set_text (GTK_LABEL(shared_page_stats_label), UTF8_SHORT_PRINTF("%s- %s", _(" Shared "), txt2));

	g_free (txt2);
}

/******************************************************************************
 *
 *   shared_page_create_stats_label
 *
 */

static GtkWidget *
shared_page_create_stats_label (void)
{
	GtkWidget	*hbox, *emptylabel;

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show(hbox);

	emptylabel = gtk_label_new(" ");
	gtk_widget_show(emptylabel);

	shared_page_stats_label = gtk_label_new (UTF8_SHORT_PRINTF("%s",_(" Shared ")));
	gtk_label_set_justify (GTK_LABEL(shared_page_stats_label), GTK_JUSTIFY_LEFT);
	gtk_widget_show(shared_page_stats_label);

	gtk_box_pack_start (GTK_BOX(hbox), shared_page_stats_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), emptylabel, TRUE, TRUE, 0);

	if (!opt_get_bool(OPT_GUI_NO_GREEN))
		set_style_recursively (hbox, style_notebook_label);

	return hbox;
}


/******************************************************************************
 *
 *  shared_page_create
 *
 *
 */

GtkWidget *
shared_page_create (void)
{
	GtkWidget *vbox, *statslabel;

	vbox = gtk_vbox_new(FALSE,0);

	gtk_widget_show(vbox);

	statslabel = shared_page_create_stats_label();

	gtk_box_pack_start (GTK_BOX(vbox), statslabel, FALSE, FALSE, 0);

	spanedwin = gtk_hpaned_new ();

	gtk_widget_show (spanedwin);

	gtk_paned_add2 (GTK_PANED(spanedwin), shared_files_create_view  () );

	gtk_paned_add1 (GTK_PANED(spanedwin), shared_dirs_create_view () );

	gtk_paned_set_position (GTK_PANED(spanedwin), opt_get_int(OPT_GUI_PANE_POS_SHARED_PAGE));

	gtk_box_pack_start (GTK_BOX(vbox), spanedwin, TRUE, TRUE, 0);

	return vbox;
}


