/* systray.c
*
* Systray support for the gui
*
* By Axel C. <tautologist@users.sourceforge.net>
*
*	TODO:
*		- add a proper icon for the systray widget.
*		- doesn't look extremely well. Perhaps try a different layout.
*		- KDE support (kde panel bug?)
*		- switch tooltip etc. to overnet if it's overnet...!
*
*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "global.h"
#include "eggtrayicon.h"
#include "icons.h"
#include "options.h"
#include "systray.h"


#if ((GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 2))

/* no systray stuff possible in gtk+-2.0, because that stuff depends on gdk-stuff which is only in 2.2 */

void         gui_systray_set_down_speed (gfloat down_speed)   { return; }
void         gui_systray_set_up_speed (gfloat up_speed)  { return; }
int          gui_systray_init (void) { return 0; }
void         gui_systray_set_status (const char *network, const char *server) { return; }

/*
	g_print ("*** WARNING: Systray does not work in GTK+ versions < 2.2.0. Systray icon will not be available.\n");
	g_print ("***          If you want the systray icon, consider upgrading your GTK+ packages.\n");
*/

#else

#	ifdef GUI_DISABLE_SYSTRAY

/* systray stuff disabled (probably because X headers not found) */

void         gui_systray_set_down_speed (gfloat down_speed)   { return; }
void         gui_systray_set_up_speed (gfloat up_speed)  { return; }
int          gui_systray_init (void) { return 0; }
void         gui_systray_set_status (const char *network, const char *server) { return; }


#	else

/***************  systray code for gtk+ >= 2.2 follows *****************************************************/

static GtkWidget     *gui_tray_icon = NULL;               /* will carbonated beverage beat me for this? */
static GtkWidget     *gui_tray_img = NULL;                /* systray icon */
static GtkWidget     *gui_tray_icon_speed_up = NULL;      /* up speed label */
static GtkWidget     *gui_tray_icon_speed_down = NULL;    /* down speed label */
static GtkTooltips   *gui_tray_icon_tooltips = NULL;

static gboolean       gui_tray_icon_borked = TRUE;        /* If TRUE, all systray calls will be foo */



static void
gui_tray_icon_onDestroy (GtkWidget *icon, gpointer bah)
{
	g_print ("%s\n", __FUNCTION__);

	gui_tray_icon_borked = TRUE;

}



/**
 * gui_systray_check_gnome ()
 *
 * Check if the gui is running under gnome, because this currently doesnt work well in KDE.
 *
 **/

static gboolean
gui_systray_check_gnome (void)
{
	gchar *env = (gchar *)g_getenv ("GNOME_DESKTOP_SESSION_ID");

	if (env)
		return TRUE;

	return FALSE;
}


void
gui_systray_set_down_speed (gfloat down_speed)
{
	gchar	*buf;

	if (gui_tray_icon_borked == TRUE)
		return;

	buf = g_strdup_printf ("<span size='small'>%3.1f kB/s</span>", down_speed);
	gtk_label_set_markup (GTK_LABEL (gui_tray_icon_speed_down), buf);
	g_free (buf);
}

void
gui_systray_set_up_speed (gfloat up_speed)
{
	gchar    *buf;

	if (gui_tray_icon_borked == TRUE)
		return;

	buf = g_strdup_printf ("<span size='small'>%3.1f kB/s</span>", up_speed);

	gtk_label_set_markup (GTK_LABEL (gui_tray_icon_speed_up), buf);

	g_free (buf);
}


void
gui_systray_set_status (const char *network, const char *server)
{
	char	*buf;

	if (gui_tray_icon_borked == TRUE)
		return;

	buf = g_strdup_printf ("%s: %s", network, server);

	if (gui_tray_icon_tooltips)
		gtk_tooltips_set_tip (gui_tray_icon_tooltips, gui_tray_icon, buf, "");

	g_free (buf);
}


int
gui_systray_init (void)
{
	GdkPixbuf		*icon;
	GtkWidget		*frame;
	GtkWidget		*hbox;
	GtkWidget		*speed_vbox;

	GtkWidget		*down_hbox, *down_img;
	GtkWidget		*up_hbox, *up_img;

	if (!opt_get_bool(OPT_GUI_USE_APPLET_IF_SUPPORTED))
	{
		gui_tray_icon_borked = TRUE;
		return -1;
	}

	if (gui_systray_check_gnome () == FALSE)
	{
		g_print ("*** WARNING: systray icon only works currently in GNOME 2. Systray icon not available.\n");
		g_print ("***          Fixes for this are welcome.\n");
		g_print ("***          If you are actually running GNOME 2, but don't use gdm, try setting\n");
		g_print ("***          environment variable GDM_DESKTOP_SESSION_ID to some random value before\n");
		g_print ("***          running the GUI\n");

		gui_tray_icon_borked = TRUE;

		return -1;
	}


	icon = gdk_pixbuf_scale_simple (get_icon (ICON_ED2K_GTK_GUI), 24, 24, GDK_INTERP_BILINEAR);
	gui_tray_img = gtk_image_new_from_pixbuf (icon);

	gui_tray_icon = GTK_WIDGET (egg_tray_icon_new ("eDonkey2000"));

	if (!gui_tray_icon)
	{
		g_print (_("*** WARNING *** Unable to create systray icon. Icon will not be usable\n"));
		gui_tray_icon_borked = TRUE;
		return -1;
	}

	frame = gtk_frame_new (NULL);
	hbox = gtk_hbox_new (FALSE, 0);
	speed_vbox = gtk_vbox_new (FALSE, 2);

	#warning *** We leak 2 pixbufs here. Fix it later

	down_img = gtk_image_new_from_pixbuf ( gdk_pixbuf_scale_simple ( get_icon (ICON_TAB_DOWNLOADS),
	                                       16,
	                                       16,
	                                       GDK_INTERP_BILINEAR));

	up_img =   gtk_image_new_from_pixbuf ( gdk_pixbuf_scale_simple ( get_icon (ICON_TAB_UPLOADS),
	                                       16,
	                                       16,
	                                       GDK_INTERP_BILINEAR));


	gui_tray_icon_speed_up = gtk_label_new ("");
	gtk_label_set_use_markup (GTK_LABEL (gui_tray_icon_speed_up), TRUE);
	gui_tray_icon_speed_down = gtk_label_new ("");
	gtk_label_set_use_markup (GTK_LABEL (gui_tray_icon_speed_down), TRUE);


	down_hbox = gtk_hbox_new (FALSE, 0);
	up_hbox = gtk_hbox_new (FALSE, 0);

	gtk_box_pack_start (GTK_BOX (down_hbox), down_img, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (down_hbox), gui_tray_icon_speed_down, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (up_hbox), up_img, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (up_hbox), gui_tray_icon_speed_up, TRUE, TRUE, 0);



	gtk_box_pack_start (GTK_BOX (speed_vbox), down_hbox, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (speed_vbox), up_hbox, FALSE, FALSE, 0);


	gtk_container_add (GTK_CONTAINER (gui_tray_icon), frame);
	gtk_container_add (GTK_CONTAINER (frame), hbox);
	gtk_box_pack_start (GTK_BOX (hbox), gui_tray_img, TRUE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (hbox), speed_vbox, TRUE, TRUE, 2);

	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);

	gtk_widget_show_all (gui_tray_icon);

	gtk_widget_realize (gui_tray_icon);
	gtk_widget_show (gui_tray_icon);

	gui_tray_icon_tooltips = gtk_tooltips_new ();
	gtk_tooltips_enable (gui_tray_icon_tooltips);

	gtk_tooltips_set_tip (gui_tray_icon_tooltips, gui_tray_icon, "eDonkey2000", "");

	gui_tray_icon_borked = FALSE;

	
	g_signal_connect (G_OBJECT (gui_tray_icon), "destroy", G_CALLBACK (gui_tray_icon_onDestroy), NULL);

	return 0;
}

#	endif /* ifdef GUI_DISABLE_SYSTRAY */

#endif /* if ((GTK_MAJOR_VERSION == 2) && (GTK_MINOR_VERSION < 2)) else ... */


