/***************************************************************************
 *
 *   uploads_page.c
 *
 *   populates upload page in notepad 
 *
 *   (c) 2001-2003 Tim-Philipp Muller < t.i.m at orange.net >
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "colors.h"
#include "global.h"
#include "gui_flags.h"
#include "options.h"
#include "misc_strings.h"
#include "uploads.h"
#include "uploads_page.h"
#include "uploadstats.h"

/* variables */

gfloat               total_uploaded = 0.0;
time_t               stats_last_cleared = 0;
time_t               seconds_upload_was_nonzero = 1; /* let's not divide by zero */

static GtkWidget	*ulpanedwin2 = NULL;
static GtkWidget	*uploads_page_stats_label = NULL;

/* functions */

static GtkWidget    *uploads_page_create_stats_label (void);

/******************************************************************************
 *
 *   uploads_page_set_people_on_queue
 *
 ***/

guint
uploads_page_get_panedwin_separator_position (void)
{
	g_return_val_if_fail (ulpanedwin2 != NULL, 50);
	return (guint) ((GTK_VPANED(ulpanedwin2)->paned).child1_size);
}


/******************************************************************************
 *
 *   uploads_page_set_people_on_queue
 *
 ***/

void
uploads_page_set_people_on_queue (guint num)
{
	g_return_if_fail (uploads_page_stats_label!=NULL);

	gtk_label_set_text (GTK_LABEL(uploads_page_stats_label),
	                    UTF8_SHORT_PRINTF(" Uploads - %u on queue ", num));
}


/******************************************************************************
 *
 *   uploads_page_create_stats_label
 *
 ***/

static GtkWidget *
uploads_page_create_stats_label (void)
{
	GtkWidget	*hbox, *emptylabel;

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_show(hbox);

	emptylabel = gtk_label_new(" ");
	gtk_widget_show(emptylabel);

	uploads_page_stats_label = gtk_label_new (UTF8_SHORT_PRINTF("%s",_(" Uploads ")));
	gtk_label_set_justify (GTK_LABEL(uploads_page_stats_label), GTK_JUSTIFY_LEFT);
	gtk_widget_show(uploads_page_stats_label);

	gtk_box_pack_start (GTK_BOX(hbox), uploads_page_stats_label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), emptylabel, TRUE, TRUE, 0);

	g_object_set_data_full (G_OBJECT(uploads_page_stats_label), "foo1",
	                        &uploads_page_stats_label, (GDestroyNotify)g_nullify_pointer);

	if (!opt_get_bool(OPT_GUI_NO_GREEN))
		set_style_recursively (hbox, style_notebook_label);

	return hbox;
}


/******************************************************************************
 *
 *   uploads_page_create
 *
 ***/

GtkWidget *
uploads_page_create (void)
{
	GtkWidget *vbox, *ulstatslabel;

	vbox = gtk_vbox_new(FALSE,0);

	gtk_widget_show(vbox);

	ulstatslabel = uploads_page_create_stats_label();

	gtk_box_pack_start (GTK_BOX(vbox), ulstatslabel, FALSE, FALSE, 0);

	ulpanedwin2 = gtk_vpaned_new ();

	g_object_set_data_full (G_OBJECT(ulpanedwin2), "foo1",
	                        &ulpanedwin2, (GDestroyNotify)g_nullify_pointer);

	gtk_widget_show (ulpanedwin2);

	gtk_paned_add1 (GTK_PANED(ulpanedwin2), uploads_create_store_model_and_view());

	gtk_paned_add2 (GTK_PANED(ulpanedwin2), uploadstats_create_store_model_and_view());

	gtk_paned_set_position (GTK_PANED(ulpanedwin2), opt_get_int(OPT_GUI_PANE_POS_UPLOADS_PAGE));

	gtk_box_pack_start (GTK_BOX(vbox), ulpanedwin2, TRUE, TRUE,0);

	return vbox;
}



